class StringHelper(object):
    # run the shell command and print the output if is in debug mode
    @staticmethod
    def replace_placeholder(to_change_string, placeholder, value):
        return to_change_string.replace(placeholder, value)



from . import abstract_resource


class ServiceResource(abstract_resource.AbstractResource):
    def __init__(self, properties, global_variables=None):
        super(ServiceResource, self).__init__(properties, global_variables)
        self.ensure = properties.get('ensure', None)

    @property
    def ensure(self):
        return self.__ensure

    @ensure.setter
    def ensure(self, ensure):
        self.__ensure = ensure

    # compare the service status with the given status
    def is_current_status_right(self):
        if self.ensure == 'restarted':
            status = 'running'
        else:
            status = self.ensure

        return True if self.current_status() == status else False

    # check a service status e.g. running, dead
    def current_status(self):
        # get service status
        status = self._run_shell_command('systemctl show -p SubState ' + self.name)

        # extract the word after = from status e.g. SubState=running
        return status.partition('=')[2]

    def run(self):
        if self.is_current_status_right() is False:
            # given status does not match with the current status of the service
            if self.ensure == 'running':
                # try to start the service
                self._run_shell_command('service ' + self.name + ' start')
            elif self.ensure == 'dead':
                # try to stop the service
                self._run_shell_command('service ' + self.name + ' stop')
            elif self.ensure == 'restarted':
                # try to restart the service
                self._run_shell_command('service ' + self.name + ' restart')

            # check the status again
            if self.is_current_status_right() is False:
                # service status is still the same cannot do much about it
                raise Exception('was not able to change the service status to ' + self.ensure)

        print('*' + self.name + '* is ' + self.ensure)



import sys
from iautomate import IAutomate
import traceback
import os


def main():
    # check running as sudo:
    if not os.geteuid() == 0:
        sys.exit("\nYou must run this script as sudo\n")

    # check config file is provided
    try:
        config_file = sys.argv[1]
    except IndexError:
        print('Error: Config file is missing')
        return

    try:
        iautomate = IAutomate(config_file)
        iautomate.run()
    except Exception as e:
        print('Error: ' + str(e))


if __name__ == '__main__':
    sys.exit(main())



from ..helpers import package_helper
from . import abstract_resource


class PackageResource(abstract_resource.AbstractResource):
    def __init__(self, properties, global_variables=None):
        super(PackageResource, self).__init__(properties, global_variables)
        self.version = properties.get('version', None)

    @property
    def version(self):
        return self.__version

    @version.setter
    def version(self, version):
        self.__version = version

    # determine if the version needs to be appended to the command
    def __package_version(self):
        if self.version is None or self.version == 'latest':
            return ''
        else:
            return '=' + self.version

    def run(self):
        if self.action == 'install':
            if package_helper.PackageHelper.is_package_installed(self.name) is True:
                print('Ignoring installing *' + self.name + '*. It is already installed.')
            else:
                self._run_shell_command('apt-get install -y ' + self.name + self.__package_version())
        elif self.action == 'remove':
            if package_helper.PackageHelper.is_package_installed(self.name) is False:
                print('Ignoring removing *' + self.name + '*. It is not installed.')
            else:
                self._run_shell_command('apt-get purge -y --auto-remove ' + self.name)
        else:
            print('Unsupported package action')

        # after_tasks = package.get('after_tasks', None)
        # if after_tasks:
        #     print("Handle after tasks ...")
        #     self.__handle_tasks(after_tasks)



from ..helpers import string_helper
from . import abstract_resource
import os


class DirectoryResource(abstract_resource.AbstractResource):
    def __init__(self, properties, global_variables=None):
        super(DirectoryResource, self).__init__(properties, global_variables)
        self.owner = properties.get('owner', None)
        self.group = properties.get('group', None)
        self.mode = properties.get('mode', None)
        self.ensure = properties.get('ensure', None)

    # overwrite the parent get
    @property
    def name(self):
        # update the name if doc_root is presented in it
        if self.global_variables and self.global_variables.doc_root:
            return string_helper.StringHelper.replace_placeholder(self.__name, '{$doc_root}',
                                                                  self.global_variables.doc_root)
        else:
            return self.__name

    # overwrite this one too, because the getter is already overwritten
    @name.setter
    def name(self, name):
        self.__name = name

    @property
    def owner(self):
        return self.__owner

    @owner.setter
    def owner(self, owner):
        self.__owner = owner

    @property
    def group(self):
        return self.__group

    @group.setter
    def group(self, group):
        self.__group = group

    @property
    def mode(self):
        return str(self.__mode)

    @mode.setter
    def mode(self, mode):
        self.__mode = mode

    @property
    def ensure(self):
        return self.__ensure

    @ensure.setter
    def ensure(self, ensure):
        self.__ensure = ensure

    def run(self):
        if self.action == 'create':
            # check if directory does not exist
            if os.path.isdir(self.name) is not False:
                raise OSError('Cannot create the dir: ' + self.name + ' It already exists')

            # directory does not exist, create it
            self._run_shell_command('mkdir ' + self.name)

            # set owner and group
            self._run_shell_command('chown ' + self.owner + ':' + self.group + ' ' + self.name)

            # set mode
            self._run_shell_command('chmod ' + self.mode + ' ' + self.name)

            # check if the directory has been created - after changing the group to avoid permission issue
            if os.path.isdir(self.name) is not True:
                raise OSError('Unable to create the dir: ' + self.name)
        elif self.action == 'remove' and os.path.isdir(self.name) is True:
            self._run_shell_command('rm -rf ' + self.name)

            # check if the folder has been removed
            if os.path.isdir(self.name) is not False:
                raise OSError('Unable to delete directory: ' + self.name)

        # for the time being check 'present' after action
        if self.ensure == 'present' and os.path.isdir(self.name) is not True:
            raise OSError('Folder: ' + self.name + ' does not exist')



class GlobalVariables(object):
    DOC_ROOT_KEY = 'doc_root'
    DEBUG_KEY = 'debug'
    SUDO_KEY = 'sudo'

    def __init__(self, variables):
        self.sudo = variables.get(self.SUDO_KEY, None)
        self.debug = variables.get(self.DEBUG_KEY, None)
        self.doc_root = variables.get(self.DOC_ROOT_KEY, None)

    @property
    def sudo(self):
        return self.__sudo

    @sudo.setter
    def sudo(self, sudo):
        self.__sudo = sudo

    @property
    def debug(self):
        return self.__debug

    @debug.setter
    def debug(self, debug):
        self.__debug = debug

    @property
    def doc_root(self):
        return self.__doc_root

    @doc_root.setter
    def doc_root(self, doc_root):
        self.__doc_root = doc_root

    # is in debug mode
    def is_debug_mode(self):
        return True if self.debug else False

    # is sudo is enabled
    def is_sudo_enabled(self):
        return True if self.sudo else False



from abc import ABCMeta, abstractmethod

from ..helpers import shell_helper


class AbstractResource(object):
    __metaclass__ = ABCMeta

    AFTER_TASKS_KEY = 'after_tasks'

    @abstractmethod
    def __init__(self, properties, global_variables=None):
        self.properties = properties
        self.global_variables = global_variables
        self.name = properties['name']
        self.action = properties.get('action', None)
        self.sudo = properties.get('sudo', None)
        self.after_tasks = properties.get('after_tasks', None)

    @property
    def properties(self):
        return self.__properties

    @properties.setter
    def properties(self, properties):
        self.__properties = properties

    @property
    def name(self):
        return self.__name

    @name.setter
    def name(self, name):
        self.__name = name

    @property
    def action(self):
        return self.__action

    @action.setter
    def action(self, action):
        self.__action = action

    @property
    def sudo(self):
        return self.__sudo

    @sudo.setter
    def sudo(self, sudo):
        self.__sudo = sudo

    @property
    def global_variables(self):
        return self.__global_variables

    @global_variables.setter
    def global_variables(self, global_variables):
        self.__global_variables = global_variables

    @property
    def after_tasks(self):
        return self.__after_tasks

    @after_tasks.setter
    def after_tasks(self, after_tasks):
        self.__after_tasks = after_tasks

    @abstractmethod
    def run(self):
        pass

    # check if sudo is enabled
    def is_sudo_enabled(self):
        # if self.sudo exists, it overwrites the global variable
        if self.sudo is True or self.sudo is False:
            return self.sudo

        return True if self.global_variables and self.global_variables.is_sudo_enabled() is True else False

    # run the shell command and print the output if is in debug mode
    def _run_shell_command(self, command):
        # determine sudo
        sudo = 'sudo ' if self.is_sudo_enabled() is True else ''

        output = shell_helper.ShellHelper.run_command(sudo + command)
        if self.global_variables and self.global_variables.is_debug_mode() is True:
            print('* Running: ' + command)

            if output:
                print('** Output: ')
                print(output)

        # make sure there is no whitespace in the output
        return output.strip()



import json
import os
from collections import OrderedDict
from . import global_variables
from .resources import abstract_resource
from .resources import execution_resource
from .resources import file_resource
from .resources import package_resource
from .resources import service_resource
from .resources import directory_resource


class IAutomate(object):
    VARS_KEY = 'vars'
    TASKS_KEY = 'tasks'

    def __init__(self, config_file):
        self.config_file = config_file
        self.config = self.__parse_config_file()
        self.global_variables = self.config.get(self.VARS_KEY, None)

    @property
    def config_file(self):
        return self.__config_file

    @config_file.setter
    def config_file(self, config_file):
        # check if the config file exists
        if os.path.isfile(config_file) is True:
            self.__config_file = config_file
        else:
            raise OSError('Config file does not exist: ' + config_file)

    @property
    def config(self):
        return self.__config

    @config.setter
    def config(self, config):
        # check if the config file is not empty
        if config:
            self.__config = config
        else:
            raise OSError('Config cannot be empty')

    @property
    def global_variables(self):
        return self.__global_variables

    @global_variables.setter
    def global_variables(self, variables):
        # check if the config file is not empty
        self.__global_variables = global_variables.GlobalVariables(variables)

    # parse the config file which is in json
    def __parse_config_file(self):
        return json.load(open(self.config_file), object_pairs_hook=OrderedDict)

    # handle execution resource
    def __handle_execs(self, execs):
        for execution in execs:
            self.__handle_exec(execution)

    # handle execution resource
    def __handle_exec(self, execution_properties):
        # instantiate execution model and run it
        execution = execution_resource.ExecutionResource(execution_properties, self.global_variables)
        execution.run()

        if execution_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY, None):
            self.__handle_tasks(execution_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY))

    # handle package resource
    def __handle_packages(self, packages):
        for package in packages:
            self.__handle_package(package)

    # handle package resource
    def __handle_package(self, package_properties):
        # instantiate package model and run it
        package = package_resource.PackageResource(package_properties, self.global_variables)
        package.run()

        if package_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY, None):
            self.__handle_tasks(package_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY))

    # handle service resource
    def __handle_services(self, services):
        for service in services:
            self.__handle_service(service)

    # handle service resource
    def __handle_service(self, service_properties):
        # instantiate service model and run it
        service = service_resource.ServiceResource(service_properties, self.global_variables)
        service.run()

        if service_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY, None):
            self.__handle_tasks(service_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY))

    # handle file resources
    def __handle_files(self, files):
        # iterate through the files
        for file in files:
            self.__handle_file(file)

    # handle file resource
    def __handle_file(self, file_properties):
        # instantiate file model and run it
        file_resource_obj = file_resource.FileResource(file_properties, self.global_variables)
        file_resource_obj.run()

        if file_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY, None):
            self.__handle_tasks(file_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY))

    # handle directory resources
    def __handle_directories(self, directories):
        # iterate through the directories
        for directory in directories:
            self.__handle_directory(directory)

    def __handle_directory(self, directory_properties):
        # instantiate directory model and run it
        directory = directory_resource.DirectoryResource(directory_properties, self.global_variables)
        directory.run()

        if directory_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY, None):
            self.__handle_tasks(directory_properties.get(abstract_resource.AbstractResource.AFTER_TASKS_KEY))

    def __handle_tasks(self, tasks):
        # iterate through the tasks
        for task in tasks:
            self.__handle_task(task)

    def __handle_task(self, task):
        # for each task handle the sub items based on the type
        for config_type, properties in task.items():
            if config_type == 'execs':
                print('| Handling execs ...')
                self.__handle_execs(properties)
            elif config_type == 'packages':
                print('| Handling packages ...')
                self.__handle_packages(properties)
            elif config_type == 'services':
                print('| Handling services ...')
                self.__handle_services(properties)
            elif config_type == 'files':
                print('| Handling files ...')
                self.__handle_files(properties)
            elif config_type == 'directories':
                print('| Handling directories ...')
                self.__handle_directories(properties)
            else:
                # unsupported resource
                print('Unsupported resource: ' + config_type)

    # run the tasks in the config file
    def run(self):
        print('Processing the config file ...')
        self.__handle_tasks(self.config[self.TASKS_KEY])
